@@ -60,7 +60,16 @@ const float LFO_FREQ_MIN  = 0.4f;
 const float LFO_FREQ_MAX  = 40.0f;
 
 
+// maximum helper
+
+inline float synthv1_max ( float a, float b )
+{
+	return (a > b ? a : b);
+}
+
+
 // hyperbolic-tangent fast approximation
+
 inline float synthv1_tanhf ( const float x )
 {
 	const float x2 = x * x;
@@ -117,6 +126,14 @@ inline float synthv1_pow2f ( const float x )
 }
 
 
+// convert note to frequency (hertz)
+
+inline float synthv1_freq ( float note )
+{
+	return (440.0f / 32.0f) * ::powf(2.0f, (note - 9.0f) / 12.0f);
+}
+
+
 // envelope
 
 struct synthv1_env
@@ -534,7 +551,7 @@ public:
 		m_t1 = m_b3; m_b3 = (m_b2 + m_t2) * m_p - m_b3 * m_f;
 
 		m_b4 = (m_b3 + m_t1) * m_p - m_b4 * m_f;
-	//	m_b4 = m_b4 - m_b4 * m_b4 * m_b4 * 0.166667f;	-- clipping
+		m_b4 = m_b4 - m_b4 * m_b4 * m_b4 * 0.166667f;	// clipping
 
 		m_b0 = input;
 
@@ -612,9 +629,9 @@ public:
 
 protected:
 
-	virtual float evaluate(uint16_t i)
+	float evaluate(uint16_t i)
 	{
-		synthv1_ramp1::evaluate(i);
+		synthv1_ramp1::update();
 
 		const float wbal = 0.25f * M_PI * (1.0f + m_param1_v);
 
@@ -633,9 +650,9 @@ public:
 
 protected:
 
-	virtual float evaluate(uint16_t i)
+	float evaluate(uint16_t i)
 	{
-		synthv1_ramp3::evaluate(i);
+		synthv1_ramp3::update();
 
 		const float wpan = 0.25f * M_PI
 			* (1.0f + m_param1_v)
@@ -647,7 +664,26 @@ protected:
 };
 
 
-// a forward decl.
+// pressure smoother (3 parameters)
+
+class synthv1_pre : public synthv1_ramp3
+{
+public:
+
+	synthv1_pre() : synthv1_ramp3() {}
+
+protected:
+
+	float evaluate(uint16_t)
+	{
+		synthv1_ramp3::update();
+
+		return m_param1_v * synthv1_max(m_param2_v, m_param3_v);
+	}
+};
+
+
+// forward decl.
 
 class synthv1_impl;
 
@@ -666,6 +702,7 @@ struct synthv1_voice : public synthv1_list<synthv1_voice>
 	int note;									// voice note
 
 	float vel1, vel2;							// velocities to vol
+	float pre1, pre2;							// key pressure/aftertouch
 
 	float dco1_freq1, dco1_sample1;				// frequency and phase
 	float dco1_freq2, dco1_sample2;
@@ -676,8 +713,8 @@ struct synthv1_voice : public synthv1_list<synthv1_voice>
 
 	synthv1_bal dco1_bal, dco2_bal;
 
-	synthv1_filter1 dcf11, dcf12, dcf13, dcf14;	// filters
-	synthv1_filter2 dcf21, dcf22, dcf23, dcf24;
+	synthv1_filter1 dcf11, dcf12, dcf21, dcf22;	// filters
+	synthv1_filter2 dcf13, dcf14, dcf23, dcf24;
 
 	synthv1_env::State dca1_env, dca2_env;		// envelope states
 	synthv1_env::State dcf1_env, dcf2_env;
@@ -686,6 +723,8 @@ struct synthv1_voice : public synthv1_list<synthv1_voice>
 	synthv1_glide dco1_glide1, dco1_glide2;	// glides (portamento)
 	synthv1_glide dco2_glide1, dco2_glide2;
 
+	synthv1_pre dca1_pre, dca2_pre;
+
 	bool sustain;
 };
 
@@ -780,8 +819,6 @@ private:
 	synthv1_pan   m_pan1, m_pan2;
 	synthv1_ramp4 m_vol1, m_vol2;
 
-	synthv1_ramp2 m_pre1, m_pre2;
-
 	synthv1_fx_chorus   m_chorus;
 	synthv1_fx_flanger *m_flanger;
 	synthv1_fx_phaser  *m_phaser;
@@ -1193,14 +1230,6 @@ float *synthv1_impl::paramPort ( synthv1::ParamIndex index )
 }
 
 
-// convert note to frequency (hertz)
-
-inline float note_freq ( float note )
-{
-	return (440.0f / 32.0f) * ::powf(2.0f, (note - 9.0f) / 12.0f);
-}
-
-
 // handle midi input
 
 void synthv1_impl::process_midi ( uint8_t *data, uint32_t size )
@@ -1251,6 +1280,10 @@ void synthv1_impl::process_midi ( uint8_t *data, uint32_t size )
 			// balance
 			pv->dco1_bal.reset(m_dco1.balance);
 			pv->dco2_bal.reset(m_dco2.balance);
+			// pressure/aftertouch
+			pv->pre1 = pv->pre2 = 0.0f;
+			pv->dca1_pre.reset(m_def1.pressure, &m_ctl.pressure, &pv->pre1);
+			pv->dca2_pre.reset(m_def2.pressure, &m_ctl.pressure, &pv->pre2);
 			// phases
 			pv->dco1_sample1 = pv->dco1_osc1.start();
 			pv->dco1_sample2 = pv->dco1_osc2.start(*m_dco1.phase * PHASE_SCALE);
@@ -1262,28 +1295,26 @@ void synthv1_impl::process_midi ( uint8_t *data, uint32_t size )
 				+ *m_dco1.tuning * TUNING_SCALE;
 			const float detune1
 				= *m_dco1.detune * DETUNE_SCALE;
-			pv->dco1_freq1 = note_freq(freq1 - detune1);
-			pv->dco1_freq2 = note_freq(freq1 + detune1);
+			pv->dco1_freq1 = synthv1_freq(freq1 - detune1);
+			pv->dco1_freq2 = synthv1_freq(freq1 + detune1);
 			const float freq2 = float(key)
 				+ *m_dco2.octave * OCTAVE_SCALE
 				+ *m_dco2.tuning * TUNING_SCALE;
 			const float detune2
 				= *m_dco2.detune * DETUNE_SCALE;
-			pv->dco2_freq1 = note_freq(freq2 - detune2);
-			pv->dco2_freq2 = note_freq(freq2 + detune2);
+			pv->dco2_freq1 = synthv1_freq(freq2 - detune2);
+			pv->dco2_freq2 = synthv1_freq(freq2 + detune2);
 			// filters
-			const synthv1_filter1::Type type1
-				= synthv1_filter1::Type(int(*m_dcf1.type));
-			pv->dcf11.reset(type1);
-			pv->dcf12.reset(type1);
-			pv->dcf13.reset(type1);
-			pv->dcf14.reset(type1);
-			const synthv1_filter2::Type type2
-				= synthv1_filter2::Type(int(*m_dcf2.type));
-			pv->dcf21.reset(type2);
-			pv->dcf22.reset(type2);
-			pv->dcf23.reset(type2);
-			pv->dcf24.reset(type2);
+			const int type1 = int(*m_dcf1.type);
+			pv->dcf11.reset(synthv1_filter1::Type(type1));
+			pv->dcf12.reset(synthv1_filter1::Type(type1));
+			pv->dcf13.reset(synthv1_filter2::Type(type1));
+			pv->dcf14.reset(synthv1_filter2::Type(type1));
+			const int type2 = int(*m_dcf2.type);
+			pv->dcf21.reset(synthv1_filter1::Type(type2));
+			pv->dcf22.reset(synthv1_filter1::Type(type2));
+			pv->dcf23.reset(synthv1_filter2::Type(type2));
+			pv->dcf24.reset(synthv1_filter2::Type(type2));
 			// envelopes
 			m_dcf1.env.start(&pv->dcf1_env);
 			m_dcf2.env.start(&pv->dcf2_env);
@@ -1328,14 +1359,23 @@ void synthv1_impl::process_midi ( uint8_t *data, uint32_t size )
 			}
 		}
 	}
+	// key pressure/poly.aftertouch
+	else if (status == 0xa0) {
+		synthv1_voice *pv = m_notes[key];
+		if (pv && pv->note >= 0) {
+			const float pre = float(value) / 127.0f;
+			pv->pre1 = *m_def1.pressure * pre;
+			pv->pre2 = *m_def2.pressure * pre;
+		}
+	}
 	// control change
 	else if (status == 0xb0) {
 		switch (key) {
 		case 0x01: {
 			// modulation wheel (cc#1)
-			const float modwheel = float(value) / 127.0f;
-			m_ctl.modwheel1 = *m_def1.modwheel * modwheel;
-			m_ctl.modwheel2 = *m_def2.modwheel * modwheel;
+			const float mod = float(value) / 127.0f;
+			m_ctl.modwheel1 = *m_def1.modwheel * mod;
+			m_ctl.modwheel2 = *m_def2.modwheel * mod;
 			break;
 		}
 		case 0x07:
@@ -1465,9 +1505,6 @@ void synthv1_impl::reset (void)
 	m_pan2.reset(m_out2.panning, &m_ctl.panning, &m_aux2.panning);
 	m_wid2.reset(m_out2.width);
 	
-	m_pre1.reset(m_def1.pressure, &m_ctl.pressure);
-	m_pre2.reset(m_def2.pressure, &m_ctl.pressure);
-
 	// flangers
 	if (m_flanger == 0)
 		m_flanger = new synthv1_fx_flanger [m_iChannels];
@@ -1527,17 +1564,6 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 	if (int(*m_lfo2.shape) != int(lfo2_wave.shape()) || *m_lfo2.width != lfo2_wave.width())
 		lfo2_wave.reset(synthv1_wave::Shape(*m_lfo2.shape), *m_lfo2.width);
 
-	m_wid1.process(nframes);
-	m_pan1.process(nframes);
-	m_vol1.process(nframes);
-
-	m_wid2.process(nframes);
-	m_pan2.process(nframes);
-	m_vol2.process(nframes);
-
-	m_pre1.process(nframes);
-	m_pre2.process(nframes);
-
 	// per voice
 
 	synthv1_voice *pv = m_play_list.next();
@@ -1572,17 +1598,14 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 			if (pv->lfo2_env.running && pv->lfo2_env.frames < ngen)
 				ngen = pv->lfo2_env.frames;
 
-			pv->dco1_bal.process(ngen);
-			pv->dco2_bal.process(ngen);
-
 			for (uint32_t j = 0; j < ngen; ++j) {
 
 				// velocities
 
 				const float vel1
-					= (pv->vel1 + (1.0f - pv->vel1) * m_pre1.value(j));
+					= (pv->vel1 + (1.0f - pv->vel1) * pv->dca1_pre.value(j));
 				const float vel2
-					= (pv->vel2 + (1.0f - pv->vel2) * m_pre2.value(j));
+					= (pv->vel2 + (1.0f - pv->vel2) * pv->dca1_pre.value(j));
 
 				// generators
 
@@ -1625,11 +1648,12 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 				const float reso1 = synthv1_sigmoid_1(*m_dcf1.reso
 					* env1 * (1.0f + *m_lfo1.reso * lfo1));
 
-				dco11 = pv->dcf11.output(dco11, cutoff1, reso1);
-				dco12 = pv->dcf12.output(dco12, cutoff1, reso1);
 				if (int(*m_dcf1.slope) > 0) { // 24db/octave
 					dco11 = pv->dcf13.output(dco11, cutoff1, reso1);
 					dco12 = pv->dcf14.output(dco12, cutoff1, reso1);
+				} else { // 12db/octave
+					dco11 = pv->dcf11.output(dco11, cutoff1, reso1);
+					dco12 = pv->dcf12.output(dco12, cutoff1, reso1);
 				}
 
 				const float env2 = 0.5f * (1.0f + vel2
@@ -1639,11 +1663,12 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 				const float reso2 = synthv1_sigmoid_1(*m_dcf2.reso
 					* env2 * (1.0f + *m_lfo2.reso * lfo2));
 
-				dco21 = pv->dcf21.output(dco21, cutoff2, reso2);
-				dco22 = pv->dcf22.output(dco22, cutoff2, reso2);
 				if (int(*m_dcf2.slope) > 0) { // 24db/octave
 					dco21 = pv->dcf23.output(dco21, cutoff2, reso2);
 					dco22 = pv->dcf24.output(dco22, cutoff2, reso2);
+				} else { // 12db/octave
+					dco21 = pv->dcf21.output(dco21, cutoff2, reso2);
+					dco22 = pv->dcf22.output(dco22, cutoff2, reso2);
 				}
 
 				// volumes
@@ -1682,6 +1707,14 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 
 			nblock -= ngen;
 
+			// voice ramps countdown
+
+			pv->dco1_bal.process(ngen);
+			pv->dco2_bal.process(ngen);
+
+			pv->dca1_pre.process(ngen);
+			pv->dca2_pre.process(ngen);
+
 			// envelope countdowns
 
 			if (pv->dca1_env.running && pv->dca1_env.frames == 0)
@@ -1740,6 +1773,16 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 				*out++ = synthv1_sigmoid(*in++);
 		}
 	}
+
+	// post-processing
+
+	m_wid1.process(nframes);
+	m_pan1.process(nframes);
+	m_vol1.process(nframes);
+
+	m_wid2.process(nframes);
+	m_pan2.process(nframes);
+	m_vol2.process(nframes);
 }
 
 
