@@ -797,6 +797,7 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	m_ui.StatusBar->showMessage(tr("Ready"), 5000);
 	m_ui.StatusBar->setModified(false);
+	m_ui.Preset->setDirtyPreset(false);
 }
 
 
@@ -849,10 +850,8 @@ void synthv1widget::paramChanged ( float fValue )
 			.arg(m_ui.StackedWidget->currentWidget()->windowTitle())
 			.arg(pKnob->toolTip())
 			.arg(pKnob->valueText()), 5000);
-		m_ui.StatusBar->setModified(true);
+		updateDirtyPreset(true);
 	}
-
-	m_ui.Preset->dirtyPreset();
 }
 
 
@@ -873,7 +872,7 @@ void synthv1widget::resetParams (void)
 	}
 
 	m_ui.StatusBar->showMessage(tr("Reset preset"), 5000);
-	m_ui.StatusBar->setModified(false);
+	updateDirtyPreset(false);
 }
 
 
@@ -900,11 +899,9 @@ void synthv1widget::swapParams ( bool bOn )
 		}
 	}
 
-	m_ui.Preset->dirtyPreset();
-
 	const bool bSwapA = m_ui.SwapParamsAButton->isChecked();
 	m_ui.StatusBar->showMessage(tr("Swap %1").arg(bSwapA ? 'A' : 'B'), 5000);
-	m_ui.StatusBar->setModified(true);
+	updateDirtyPreset(true);
 }
 
 
@@ -968,7 +965,7 @@ void synthv1widget::newPreset (void)
 	resetParamValues();
 
 	m_ui.StatusBar->showMessage(tr("New preset"), 5000);
-	m_ui.StatusBar->setModified(false);
+	updateDirtyPreset(false);
 }
 
 
@@ -1041,7 +1038,7 @@ void synthv1widget::loadPreset ( const QString& sFilename )
 	m_ui.Preset->setPreset(sPreset);
 
 	m_ui.StatusBar->showMessage(tr("Load preset: %1").arg(sPreset), 5000);
-	m_ui.StatusBar->setModified(false);
+	updateDirtyPreset(false);
 
 	QDir::setCurrent(currentDir.absolutePath());
 }
@@ -1078,7 +1075,7 @@ void synthv1widget::savePreset ( const QString& sFilename )
 	}
 
 	m_ui.StatusBar->showMessage(tr("Save preset: %1").arg(sPreset), 5000);
-	m_ui.StatusBar->setModified(false);
+	updateDirtyPreset(false);
 }
 
 
@@ -1145,4 +1142,12 @@ void synthv1widget::helpAboutQt (void)
 }
 
 
+// Dirty flag (overridable virtual) methods.
+void synthv1widget::updateDirtyPreset ( bool bDirtyPreset )
+{
+	m_ui.StatusBar->setModified(bDirtyPreset);
+	m_ui.Preset->setDirtyPreset(bDirtyPreset);
+}
+
+
 // end of synthv1widget.cpp
