@@ -31,6 +31,12 @@
 #define SYNTHV1_LV2UI_URI SYNTHV1_LV2_PREFIX "ui"
 
 
+#ifdef CONFIG_LV2_EXTERNAL_UI
+#include "lv2_external_ui.h"
+#define SYNTHV1_LV2UI_EXTERNAL_URI SYNTHV1_LV2_PREFIX "ui_external"
+#endif
+
+
 //-------------------------------------------------------------------------
 // synthv1widget_lv2 - decl.
 //
@@ -45,16 +51,29 @@ public:
 	void port_event(uint32_t port_index,
 		uint32_t buffer_size, uint32_t format, const void *buffer);
 
+#ifdef CONFIG_LV2_EXTERNAL_UI
+	void setExternalHost(LV2_External_UI_Host *external_host);
+	const LV2_External_UI_Host *externalHost() const;
+#endif
+
 protected:
 
 	// Param methods.
 	void updateParam(synthv1::ParamIndex index, float fValue) const;
 
+#ifdef CONFIG_LV2_EXTERNAL_UI
+	void closeEvent(QCloseEvent *pCloseEvent);
+#endif
+
 private:
 
 	// Instance variables.
 	LV2UI_Controller     m_controller;
 	LV2UI_Write_Function m_write_function;
+
+#ifdef CONFIG_LV2_EXTERNAL_UI
+	LV2_External_UI_Host *m_external_host;
+#endif
 };
 
 
